 /*************************************************************************
 * FILE NAME: $RCSfile: ppa_example.c,v $    COPYRIGHT (c) FREESCALE 2004 *
 * DESCRIPTION:                                     All Rights Reserved   *
 * This is a sample MPC5500 program to show a simple use of the eTPU PPA  *
 * function. The ETPU QOM function is used to provide a stumulus for the  *
 * PPA function to measure. An extaernal hardware connection is required  *
 * between the QOM channel and the PPA channel.                           *
 *========================================================================*
 * ORIGINAL AUTHOR: Geoff Emerson (r47354)                                *
 * $Log: ppa_example.c,v $
 * Revision 1.2  2004/12/14 11:04:13  r47354
 * - Use ETPU_INPUT_PAD_CONFIG and ETPU_OUTPUT_PAD_CONFIG
 * - Change SampleTime to 0x100000
 * - Fill result array with background of 0xFFFFFF
 *
 * Revision 1.1  2004/12/09 14:38:31  r47354
 * Updates as per QOM API rel_2_1
 *
 *........................................................................*
 * 0.1   G.Emerson    27/Apr/04    Initial version.                       *
 * 0.2                             Updated for new build structure.       *
/**************************************************************************/
#include "mpc5554.h"            /* mpc5554 register definitions */
#include "mpc5554_vars.h"       /* Variables specific to mpc5554 */
#include "etpu_util.h"          /* useful utility routines */
#include "fs_gpio.h"            /* GPIO API for the MPC5500 family */
#include "etpu_set1.h"          /* eTPU standard function set 1 */
#include "etpu_qom.h"           /* eTPU QOM API */
#include "etpu_ppa.h"           /* eTPU PPA API */

/* User written include files */
#include "ppa_example.h"        /*include application specific defines. */

union etpu_events_array my_event_array_QOMA[QOMA_array_size] =
  { (0x010000 << 1) + FS_ETPU_PIN_HIGH,
    (0x008000 << 1) + FS_ETPU_PIN_LOW,
    (0x004000 << 1) + FS_ETPU_PIN_HIGH,
    (0x002000 << 1) + FS_ETPU_PIN_LOW
};

union etpu_events_array my_event_array_QOMLO[QOMLO_array_size] =
  { (0x001000 << 1) + FS_ETPU_PIN_LOW,
    (0x001000 << 1) + FS_ETPU_PIN_LOW,
    (0x001000 << 1) + FS_ETPU_PIN_LOW,
    (0x001000 << 1) + FS_ETPU_PIN_LOW,
    (0x001000 << 1) + FS_ETPU_PIN_LOW,
    (0x001000 << 1) + FS_ETPU_PIN_LOW
};
#define ETPU_INPUT_PAD_CONFIG  FS_GPIO_PRIMARY_FUNCTION + \
                               FS_GPIO_INPUT_BUFFER_ENABLE

#define ETPU_OUTPUT_PAD_CONFIG FS_GPIO_PRIMARY_FUNCTION + \
                               FS_GPIO_MAXIMUM_SLEW_RATE + \
                               FS_GPIO_OPEN_DRAIN_DISABLE + \
                               FS_GPIO_INPUT_BUFFER_ENABLE + \
                               FS_GPIO_OUTPUT_BUFFER_ENABLE

uint32_t *fs_free_param;

main ()
{
    int32_t error_code;
    vuint16_t *upper_QOM_SCR16; /* pointers to eTPU QOM SCR */
    vuint32_t *upper_QOM_SCR32;
    vuint16_t *upper_PPA_SCR16; /* pointers to eTPU PPA SCR */
    vuint32_t *upper_PPA_SCR32;
    uint32_t *QOM_ram_ptr;      /**pointer to QOM ram */
    uint32_t *QOM_last_match_ptr;
    int32_t tcr1_now;           /*store for read tcr value*/
    int32_t n;
    uint8_t this_status;      /* PPA status */
    uint32_t accum_result;
    uint32_t my_result_accum_array[12]; /* PPA result array */

    upper_QOM_SCR32 = &ETPU.CHAN[QOM0].SCR.R;
    upper_QOM_SCR16 = (vuint16_t *) upper_QOM_SCR32;
    upper_PPA_SCR32 = &ETPU.CHAN[PPA0].SCR.R;
    upper_PPA_SCR16 = (vuint16_t *) upper_PPA_SCR32;

    FMPLL.SYNCR.R = 0x06000000; /* MFD = 0b110, RFD=0b000, 8MHz*16=128MHz */

    /* initialize eTPU hardware */
    fs_etpu_init (my_etpu_config, (uint32_t *) etpu_code,
                  sizeof (etpu_code), (uint32_t *) etpu_globals,
                  sizeof (etpu_globals));

    /* enable all timebases */
    fs_timer_start ();

    for (n = 0; n < 12; n++)
      {
          my_result_accum_array[n] = 0xFFFFFF;
          /* fill result array with 1's
             to be over-written */

      }

/* configure the pins on the MPC5554*/
/* QOM pin */
    fs_gpio_config (FS_GPIO_ETPUA8, ETPU_OUTPUT_PAD_CONFIG );

/* PPA pin */
    fs_gpio_config (FS_GPIO_ETPUA10, ETPU_INPUT_PAD_CONFIG );

    *upper_QOM_SCR16 = 0xC0C0;  /* clear QOM DMA/interrupt service request */
    *upper_PPA_SCR16 = 0xC0C0;  /* clear PPA DMA/interrupt service request */

    /* init QOM channel Low using QOM fn */
    error_code =
        fs_etpu_qom_init (QOM0,
                          FS_ETPU_PRIORITY_MIDDLE,
                          FS_ETPU_QOM_SINGLE_SHOT,
                          FS_ETPU_TCR1,
                          FS_ETPU_QOM_INIT_PIN_LOW,
                          FS_ETPU_QOM_IMMEDIATE,
                          (uint32_t *) 0,
                          7, 
                          QOMLO_array_size, 
                          my_event_array_QOMLO);
                          
    while (ETPU.CHAN[QOM0].SCR.B.CIS == 0){};
    /*wait for QOM to finish */
    
    *upper_QOM_SCR16 = 0xC0C0;
    /* clear DMA/interrupt service request */

    QOM_ram_ptr = fs_etpu_data_ram (QOM0);
    QOM_last_match_ptr =
        (uint32_t *) ((uint32_t) QOM_ram_ptr +
         FS_ETPU_QOM_LAST_MATCH_TIME_OFFSET - 1);
    /* get address to be used as reference for the QOM fn */

/*initialise the channel for PPA */
    error_code =
        fs_etpu_ppa_init (PPA0,
                          FS_ETPU_PRIORITY_MIDDLE,
                          FS_ETPU_PPA_INIT_TCR1,
                          FS_ETPU_PPA_HIGH_PULSE, 
                          2, 
                          0x100000);

    tcr1_now = ETPU.TB1R_A.R;

    *QOM_last_match_ptr = tcr1_now + 0x4000;
    /* program reference for QOM fn */

/*initialise the channel for QOM */
    error_code =
        fs_etpu_qom_init (QOM0,
                          FS_ETPU_PRIORITY_MIDDLE,
                          FS_ETPU_QOM_LOOP,
                          FS_ETPU_TCR1,
                          FS_ETPU_QOM_INIT_PIN_LOW,
                          FS_ETPU_QOM_USE_REF_ADDRESS,
                          (uint32_t *) QOM_last_match_ptr,
                          12, 
                          QOMA_array_size, 
                          my_event_array_QOMA);

    if (error_code != 0)
        return (QOM0 + 1);
        
    for (n = 0; n < 12; n++)
      {
          while (ETPU.CHAN[PPA0].SCR.B.CIS == 0){};
          /*wait for PPA0 to finish */

          this_status = fs_etpu_ppa_get_accumulation (PPA0, &accum_result);

          my_result_accum_array[n] = accum_result;

          *upper_PPA_SCR16 = 0xC0C0;    
          /* clear DMA/interrupt service request */
      }
    while (ETPU.CHAN[QOM0].SCR.B.CIS == 0){};
    /*wait for QOM to finish */
    
    *upper_QOM_SCR16 = 0xC0C0;  /* clear DMA/interrupt service request */
    
idle:
    for (;;);  /* All done. */
    
/* Inpection of my_result_accum_array should show 12 values of 0xA000 */

/* The QOM channel was configured to output the following pulse 12 times

              ___________        ___                                             
             |           |      | 0 |                                    
             | Ox8000    |      | x |                                             
_____________|           |______| 2 |                                          
                                  0
                                  0

The PPA channel is configured to accumulate the TCR1 durations of 2 high pulses.
Each time 2 high pulses are measured a channel interrupt bit us set by the eTPU 
PPA function.When this happens the host software records the acummulated amounts
in a result array. Since there are 12 double high pulses generatad the array 
contains 12 values of 0x8000 + 0x2000 = 0xA000.
*/

                                                                         
}